/* 
 * snmdbCmd.c - Interface snmdb command to Tcl.
 * 
 * Author:	Damir Delija
 * Date:	Fri Aug 19 1996
 */

#include <stdio.h>
#include <tcl.h>


#include <netmgt/netmgt_db.h>

/* ***************************************************************** */

int tclsnmdb_add ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_add_agent ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_add_connection ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_add_alias ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_add_to_view ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_delete ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_delete_agent ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_delete_color ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_delete_connection ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_delete_alias ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_delete_from_view ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_enumerate_agents ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_enumerate_aliases ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_enumerate_connections ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_enumerate_elements ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_enumerate_views ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_free_enumeration_handle ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_free_list ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_get_agent ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_get_color ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_get_element_glyph ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_get_element_type ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_get_next_element ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_get_property ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_get_view ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_init_buffer ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_free_buffer ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_lock ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_open ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_read ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_save_element ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_set_color ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_set_property ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_unlock ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_update ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_console_load ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_console_reload ( void *, Tcl_Interp *, int , char **);
int tclsnmdb_console_save_components ( void *, Tcl_Interp *, int , char **);

/* ***************************************************************** */
void *bufferHandleTable=NULL;
void *enumHandleTable=NULL;
/* ***************************************************************** */

/*aditional parse structure */
typedef  struct {
    char *cmd;			/* command name */
    int (*f)();			/* exec function */
    int minargs, maxargs;	/* number of args min and max */
    int subcmds;
    char *usage;		/* help message */
} cmdOptions;

static cmdOptions subcmdVec[] = {
/*
console_reload
*/
{"open", tclsnmdb_open, 0, 1, 0, "[dbname]" },
{"lock", tclsnmdb_lock, 0, 2, 0, "[wait] [clock]" },
{"unlock", tclsnmdb_unlock, 0, 0, 0, "" },
{"console_load", tclsnmdb_console_load, 1, 1, 0, "path" },
{"read", tclsnmdb_read, 1, 1, 0, "name" },
{"update", tclsnmdb_update, 1, 1, 0, "buffer" },

{"add", tclsnmdb_add, 1, 1, 0, "buffer" },
{"add_agent", tclsnmdb_add_agent, 2, 3, 0, "buffer agent [proxy]"},
{"add_alias", tclsnmdb_add_alias, 2, 2, 0, "buffer alias" },
{"add_connection", tclsnmdb_add_connection, 2,2,0, "buffer connection"},
{"add_to_view", tclsnmdb_add_to_view,4,7,0,"buffer view x y [z x1 y1]"},

{"delete", tclsnmdb_delete, 1, 1, 0, "element" },
{"delete_agent", tclsnmdb_delete_agent, 2, 2, 0, "buffer agent" },
{"delete_alias", tclsnmdb_delete_alias, 2, 2, 0, "buffer alias" },
{"delete_color", tclsnmdb_delete_color, 1, 1, 0, "buffer" },
{"delete_connection", tclsnmdb_delete_connection, 2,2,0, "buffer conn"},
{"delete_from_view", tclsnmdb_delete_from_view, 2, 2, 0, "buffer view"},

{"enumerate_elements", tclsnmdb_enumerate_elements, 0,2,0, "[view] [type]" },
{"enumerate_agents", tclsnmdb_enumerate_agents, 1, 1, 0, "buffer" },
{"enumerate_views", tclsnmdb_enumerate_views, 1, 1, 0, "buffer" },
{"enumerate_aliases", tclsnmdb_enumerate_aliases, 1, 1, 0, "buffer" },
{"enumerate_connections", tclsnmdb_enumerate_connections, 1, 1, 0, "buffer" },

{"get_next_element", tclsnmdb_get_next_element, 1, 1, 0, "handle" },
{"get_element_type", tclsnmdb_get_element_type, 1, 1, 0, "buffer" },
{"get_agent", tclsnmdb_get_agent, 2, 2, 0, "buffer agent" },
{"get_color", tclsnmdb_get_color, 1, 1, 0, "buffer" },
{"get_element_glyph", tclsnmdb_get_element_glyph,1,1,0,"element_glyph"},
{"get_view", tclsnmdb_get_view, 2, 2, 0, "buffer view" },
{"get_property", tclsnmdb_get_property, 2, 2, 0, "buffer property" },
{"set_property", tclsnmdb_set_property, 3, 3, 0, "buffer property val"},

{"save_element", tclsnmdb_save_element, 1, 2, 0, "buffer [file]" },
{"console_save_components", tclsnmdb_console_save_components, 1, 2, 0, "pth" },


{"init_buffer", tclsnmdb_init_buffer, 3, 3, 0, "name type buffer" },
{"free_buffer", tclsnmdb_free_buffer, 1, 1, 0, "buffer" },
{"free_enumeration_handle", tclsnmdb_free_enumeration_handle, 1, 1, 0, "enumeration_handle" },

{"set_color", tclsnmdb_set_color, 4, 4, 0, "buffer red green blue" }
};


/* ################################################################## */
/*
 * snmdb composite command:
 *

int snmdb_add (snmdb_buffer *);
int snmdb_add_agent	(snmdb_buffer *, char *, char *);
int snmdb_add_connection (snmdb_buffer *, char *);
int snmdb_add_alias      (snmdb_buffer *, char *);
int snmdb_add_to_view (snmdb_buffer *, char *, int, int, int, ...);
int snmdb_delete (char *);
int snmdb_delete_agent (snmdb_buffer *, char *);
int snmdb_delete_color	(snmdb_buffer *);
int snmdb_delete_connection	(snmdb_buffer *, char *);
int snmdb_delete_alias  	(snmdb_buffer *, char *);
int snmdb_delete_from_view	(snmdb_buffer *, char *);
char **snmdb_enumerate_agents (snmdb_buffer *);
char **snmdb_enumerate_aliases     (snmdb_buffer *);
char **snmdb_enumerate_connections (snmdb_buffer *);
snmdb_handle *snmdb_enumerate_elements	(char *, char *);
char **snmdb_enumerate_views (snmdb_buffer *);
void snmdb_free_enumeration_handle	(snmdb_handle *);
void snmdb_free_list (char **);
int snmdb_get_agent	(snmdb_buffer *, char *, int *, char *);
int snmdb_get_color	(snmdb_buffer *, int *, int *, int *);
char *snmdb_get_element_glyph (char *);
char *snmdb_get_element_type (snmdb_buffer *);
char *snmdb_get_next_element (snmdb_handle *);
snmdb_data snmdb_get_property (snmdb_buffer *, char *, snmdb_type *);
int snmdb_get_view (snmdb_buffer *, char *, int *, int *, int *, ...); 
int snmdb_init_buffer (char *, char *, snmdb_buffer *);
int snmdb_lock (int, int);
int snmdb_open ();
int snmdb_read (char *, snmdb_buffer *);
int snmdb_save_element (snmdb_buffer *, FILE *);
int snmdb_set_color (snmdb_buffer *, int, int, int);
int snmdb_set_property (snmdb_buffer *, char *, ...);
int snmdb_unlock ();
int snmdb_update (snmdb_buffer *);
int snmdb_console_load			(char *, int *, int *);
int snmdb_console_reload		(char *, int *, int *);
int snmdb_console_save_components	(char *, int *, int *);

*/
int
snmdbCmd( ClientData clientData, Tcl_Interp *interp, int argc, char *argv[] )
{
    int argi, subi;
    int res;
	
    int local_snm_error=0;  	
    /* Check for subcommand. */
    if ( argc < 2 )
    {
	interp->result = "wrong # args: should be \"snmdb option ...\"";
	return TCL_ERROR;
    }
    
    /* Find the subcommand. */
    for (subi = 0; subi < (sizeof subcmdVec) / (sizeof subcmdVec[0]); subi++)
    {
	if ( strcmp( subcmdVec[subi].cmd, argv[1] ) == 0 )
	{
	    /* Check arg count. */
	    if ( argc - 2 < subcmdVec[subi].minargs ||
		 argc - 2 > subcmdVec[subi].maxargs )
	    {
		Tcl_AppendResult( interp, "wrong # args: should be \"snmdb ",
				  subcmdVec[subi].cmd, " ",
				  subcmdVec[subi].usage, "\"", 0 );
		return TCL_ERROR;
	    }
	    
	    /* Call the subcommand function. */

    	    local_snm_error=snm_error;  	
	    snm_error=0;

	    if (argc>1 ) { argc--; argv++; }
	    res=(*subcmdVec[subi].f)(  clientData, interp, argc, argv ); 
	    if (res!=TCL_OK) {
		snmTclAddError (interp,  snm_error );
	    }
      	    snm_error=local_snm_error;
	    /* 
		here will come Error interpretation routine!
	    */

		return res;
	}
    }

    /* If we get here, the option doesn't match. */
    Tcl_AppendResult( interp, "bad option \"", argv[1], "\": should be ", 0 );
    for (subi = 0; subi < (sizeof subcmdVec) / (sizeof subcmdVec[0]); subi++)
	Tcl_AppendResult( interp, (subi > 0 ? ", " : ""),
			   subcmdVec[subi].cmd, 0 );

    return TCL_ERROR;
}

/* ################################################################## */

/* Initialize the package.  */

void Snmdbtcl_Init( Tcl_Interp *interp ) 
{
	if (tableInit(
		&bufferHandleTable,"bfdb", sizeof(snmdb_handle)
		)==TCL_ERROR) { return; }

	if (tableInit(
		&enumHandleTable,"enumdb", sizeof(snmdb_handle)
		)==TCL_ERROR) { return; }

	Tcl_CreateCommand( interp, "snmdb", snmdbCmd,
                       (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL );
}

/* ################################################################## */
