/* Copyright 1994 Sun Microsystems, Inc. All Rights Reserved. */
/**************************************************************************
 *	File:	db_add.c
 *
 * Corresponds to chapter 8.6: Adding a New Element Instance into 
 * the data base
 *
 **************************************************************************/

/* Please initialize the string "my_view" to a view existing 
 * in your SNM database
 */

#include <sys/types.h>
#include <rpc/types.h>
#include <stdio.h>
#ifdef SVR4
#include <sys/utsname.h>
#endif
#include <netmgt/netmgt_db.h>

main()
{
snmdb_buffer	buf;		/* cluster record buffer of element */
char	my_view[128];		/* view existing in the current database */
char	local_host[256];	/* local host name */

#ifdef SVR4
struct utsname ut;
#endif

	/* Initialize the name of a view existing in the database */
	(void) strcpy( my_view, "Home");

	/* Let's take the local host name as element to search 
	 * in the data base
	 */
#ifdef SVR4
	if (uname(&ut) == -1) {
		perror("uname");
		return 1;
	} else
		strcpy(local_host, ut.nodename);
#else
	if (gethostname(local_host, sizeof(local_host)) == -1) {
		perror("gethostname");
		return 1;
	}
#endif

	/* Open the database */
	if (!snmdb_open()) {
		printf("snmdb_open() failed, error = %d\n", snm_error);
		exit(1);
	}

/* The following code segment shows how to add a new element "andrew" */

	/* initialize the cluster record buffer of the element */
	if (!snmdb_init_buffer("andrew", "component.ss1", &buf)) {
		printf("snmdb_init_buffer() failed, error = %d\n",
			snm_error);
		exit(1);
	}

	/* set the property */
/*
	if (!snmdb_set_property(&buf, "User", "test machine") ||
	    !snmdb_set_property(&buf, "Location", "Building 14") ) {
		printf("snmdb_set_property() failed, error = %d\n",
			snm_error);
		exit(1);
	}
*/

	/* set the color of the element */
	snmdb_set_color(&buf, 100, 220, 0);

	/* add a non-proxy and a proxy agent */
	if (!snmdb_add_agent(&buf, "hostif", NULL) ||
	    !snmdb_add_agent(&buf, "hostperf", "andrew")) {
		printf("snmdb_add_agent() failed, error = %d\n",
			snm_error);
		exit(1);
	}

	/* add a connection to another element 
	 * Note: the element you want to connect should be 
	 * a valid element name that exists in the database
	 */
	if (!snmdb_add_connection(&buf, local_host)) { 
		printf("snmdb_add_connection() failed, error = %d\n",
			snm_error);
		exit(1);
	}


	/* add the element 'andrew' into the view "my_view"  */
	/* be sure that the view exists else the element won't be added */
	if (!snmdb_add_to_view(&buf, my_view, 10, 50, 0, 0, 0)) {
		printf("snmdb_add_to_view() failed, error = %d\n",
			snm_error);
		exit(1);
	}


	/* now we are ready to add the element to the database */
	snmdb_lock(TRUE, FALSE);
	if (!snmdb_add(&buf)) {
		printf("snmdb_add() failed, error = %d\n", snm_error);
		if (snm_error == SNMDB_DUPLICATE_NAME) {
			printf("element 'andrew' already exists \n");
		}
		snmdb_unlock();
		exit(1);
	}
	/* element added successfully */
	snmdb_unlock();
	printf("Added element 'andrew' in view '%s' \n", my_view);


}
	

