/* Copyright 1994 Sun Microsystems, Inc. All Rights Reserved. */
/**************************************************************************
 *	File:	db_view.c
 *
 * Corresponds to chapter 8.5.1: Retrieving Info for a Single Element
 *			  8.5.2: Retrieving Elements of a Given Type
 *
 **************************************************************************/

/* Please initialize the string "my_view" to a view existing 
 * in your SNM database
 */

#include <sys/types.h>
#include <rpc/types.h>
#include <stdio.h>
#ifdef SVR4
#include <sys/utsname.h>
#endif
#include <netmgt/netmgt_db.h>

main()
{


snmdb_buffer	buf;		/* cluster record buffer of element */
int	red, green, blue;	/* color value */
int	isproxy;		/* indicator - whether agent is proxy */
int	x,y,z;			/* x, y, z coordinates in a view */
char	name[128];		/* name buffer */
char	local_host[256];	/* local host name */
char	my_view[128];		/* view existing in the current database */

char 	**names;		/* pointer to the enumerated list */
char	**str;			/* temp pointer */

snmdb_handle	*handle;	/* pointera to the enumerated list */
char	*str2;			/* temp pointer */
#ifdef SVR4
struct utsname ut;
#endif

	/* Initialize the name of a view existing in the databese */
	(void) strcpy( my_view, "Home");

	/* Let's take the local host name as element to search 
	 * in the data base
	 */
#ifdef SVR4
	if (uname(&ut) == -1) {
		perror("uname");
		return 1;
	} else
		strcpy(local_host, ut.nodename);
#else
	if (gethostname(local_host, sizeof(local_host)) == -1) {
		perror("gethostname");
		return 1;
	}
#endif

	/* Open the database */
	if (!snmdb_open()) {
		printf("snmdb_open() failed, error = %d\n", snm_error);
		exit(1);
	}

/* 1st part */
	printf("\n\nPart I \n\n");

/* The following code segment shows how to retrieve information on 
 * a single element 
 * The element chosen as an example is the local host
 */
	/* first, we load the element record into the cluster record buffer */
	if (!snmdb_read(local_host,&buf)) {	/* read error */
		printf("snmdb_read() failed, error = %d\n", snm_error);
		exit(1);
	}

	/* get the type of the element */
	printf("The type of the element %s is %s\n",
		local_host, (char *) snmdb_get_element_type(&buf));

	/* get the value of property 'User', 'Location', and 'Description */
	printf("The 'User' field = %s\n",
		snmdb_get_property(&buf, "User", 0).db_string );
	printf("The 'Location' field = %s\n",
		snmdb_get_property(&buf, "Location", 0).db_string );
	printf("The 'Description' field = %s\n",
		snmdb_get_property(&buf, "Description", 0).db_string );
	

	/* get the color of it */
	/* colors are represented in RGB numbers, snmdb_get_color()
	 * will return the RGB numbers of the element glyph
	 */
	if ( snmdb_get_color(&buf, &red, &green, &blue) )
		printf( "Color values are: %d, %d, %d\n", red, green, blue);
	else
		printf("There is no color defined for the element\n");

	/* get agent information for agent 'hostperf' */
	if (snmdb_get_agent(&buf, "hostperf", &isproxy, name)) {
		if (isproxy)
			printf("hostperf is proxy, proxy name = %s\n", name);
		else
			printf("hostperf is not proxy\n");
	} else {
		printf("agent 'hostperf' is not running on %s \n", local_host);
	}

	/* get the location of the element in a certain view */
	if (snmdb_get_view(&buf, my_view, &x, &y, &z, NULL, NULL)) {
		printf("Location of '%s' in view '%s' is :\n",
			local_host, my_view);
		printf(" x = %d, y = %d, z = %d\n", x, y, z);
	} else {
		printf("Element is not in view '%s', error = %d\n",
			my_view, snm_error);
        }


/* 2nd part */
	printf("\n\nPart II \n\n");

/* The following code segment shows how to get all the agents that 
 * apply to the target element, how to get all the element names that 
 * the target element is connected to, and how to get all the views in 
 * which the target element exist.
 */

	/* get all the agent names that run on (or apply to) the element */
	if ((names = snmdb_enumerate_agents(&buf)) == NULL) {
		printf("Error: unable to enumerate agents, error = %d\n",
			snm_error);
		if (snm_error == SNMDB_NO_AGENT_APPLY) {
			printf("  Element does not have any agent \n");
		} else {
			exit(1);
		}
	} else {
		/* got the enumerated list successfully, print the list */
		for (str = names; *str; str++)
			printf("agent `%s' found\n", *str);
		/* remember to free the space used */
		snmdb_free_list(names);
	}

	/* get all the element names that THE element is connected to */ 
	if ((names = snmdb_enumerate_connections(&buf)) == NULL) {
		printf("Error: unable to enumerate connections, error = %d\n",
			snm_error);
		if (snm_error == SNMDB_NOT_CONNECTED_TO_ANY) {
			printf("  Element not connected to any other \n");
		} else {
			exit(1);
		}
	} else {
		/* got the enumerated list successfully, print the list */
		for (str = names; *str; str++)
		printf("connected to `%s'\n", *str);
		/* remember to free the space used */
		snmdb_free_list(names);
	}

	/* get all the view names that THE element exists in */
	if ((names = snmdb_enumerate_views(&buf)) == NULL ) {
		printf("Error: unamble to enumerate views, error = %d\n",
			snm_error);
		exit(1);
	}

	/* got the enumerated list successfully, print the list */
	for (str = names; *str; str++)
		printf("element exists in view `%s'\n", *str);
	/* remember to free the space used */
	snmdb_free_list(names);
	

/* 3rd part */
	printf("\n\nPart III \n\n");

/* The following code shows how to retrieve all element names in 
 * all views
 */
	if (!(handle = snmdb_enumerate_elements(NULL, NULL))) {
		printf("No element in any view\n");
		exit(1);
	}

	printf("Elements of all types in all views are: \n");
	while (str2 = snmdb_get_next_element(handle))
		printf("element = %s\n", str2);
	printf("end of list\n");
	
	/* remember to free the space */
	snmdb_free_enumeration_handle(handle);

/* 4th part */
	printf("\n\nPart IV \n\n");

/* The following code shows how to retrieve all element names of 
 * a given type in one particular view 
 */
	if (!(handle = snmdb_enumerate_elements(my_view, "component.ss1"))) {
		printf("No element of type 'component.ss1' in view '%s'\n",
			my_view);

	} else {

		printf("Elements of type 'component.ss1' in view '%s' are:\n",
			my_view);
		while (str2 = snmdb_get_next_element(handle))
			printf("element = %s\n", str2);
		printf("end of view\n");
	
		/* remember to free the space */
		snmdb_free_enumeration_handle(handle);
	}

}
	

