/* Copyright 1994 Sun Microsystems, Inc. All Rights Reserved. */
/**************************************************************************
 *	File:	request.c
 *
 * Corresponds to chapter 4.9: Requesting Data and Event Reports  
 * 
 *
 **************************************************************************/
#include <sys/param.h>
#include <sys/types.h>
#include <netmgt/netmgt.h>

#ifdef SVR4
#include <rpc/rpcent.h>
#else
#include <rpc/rpc.h>
#endif

#ifdef SVR4
struct timeval *
request_event( u_int count, struct timeval interval, struct timeval timeout,
    	Netmgt_thresh *threshold, char *group, char *optstring, char *key,
	u_int flags);

struct timeval *
request_data( u_int count, struct timeval interval, struct timeval timeout,
    char *group, char *optstring, char *key, u_int flags);
#else
struct timeval * request_event();
struct timeval * request_data();
#endif
bool_t set_snmp_args();


char 		agentname[NETMGT_NAMESIZ];	/* name of agent */

char		target[MAXHOSTNAMELEN];		/* name of target system */
char		agent_host[MAXHOSTNAMELEN];	/* agent hostname */
char 		event_host[MAXHOSTNAMELEN];	/* host where the event dispatcher runs*/

#define SNMP_RPC_NUM	100122


main()
{

	char	local_host[MAXHOSTNAMELEN];	/* local host name */
	u_int	count = 2;			/* report count */
	struct timeval interval;		/* report interval */
	struct timeval timeout;			/* request RPC timeout */
	Netmgt_thresh	threshbuf;		/* prebuilt threshold buffer*/
	char	group[NETMGT_NAMESIZ];		/* group name */
	char	*key = (char *) NULL;		/* not needed as the group
						 * is not a table */
	char 	*optstring= (char *) NULL;	/* optional string */
	u_int	flags = 0;			/* request option flags */
	u_short	value = 0;			/* for operator CHANGED content
						 * is ignored but param needed*/
	

	netmgt_debug = 1;


	interval.tv_sec = (long) 0; /* default interval */
	interval.tv_usec = (long) 0;

	timeout.tv_sec = NETMGT_TIMEOUT;
	timeout.tv_usec = 0;

	if (gethostname(local_host, sizeof(local_host)) == -1) {
		perror("gethostname");
		return 1;
	}

	(void) strcpy(event_host, local_host);

	/* Agent and target hosts must be the same for non-proxy agents */
	(void) strcpy(agent_host, local_host);
	(void) strcpy(target, local_host);


#ifdef SNMP_AGENT
	(void) strcpy(agentname, "sun-snmp");
	(void) strcpy(group, "system");
#else
	(void) strcpy(agentname, "ping");
	(void) strcpy(group, "stats");
#endif


#ifdef REQUEST_EVENT_REPORT

#ifdef SNMP_AGENT
	/* fill the threshold buffer  */
	(void) strcpy( threshbuf.name, "sysUpTime");
	threshbuf.type = NETMGT_TIMETICKS;
	threshbuf.relop = NETMGT_CHANGED;
	threshbuf.thresh_len = sizeof(int);
	threshbuf.thresh_val = (caddr_t)&value;
	threshbuf.priority = NETMGT_LOW_PRIORITY;
	 
#else	/* not SNMP_AGENT */

	/* fill the threshold buffer  */
	/* find out when more than 0 packets have been sent */
	(void) strcpy( threshbuf.name, "sent");
	threshbuf.type = NETMGT_GAUGE;
	threshbuf.relop = NETMGT_GT;
	threshbuf.thresh_len = sizeof(int);
	threshbuf.thresh_val = (caddr_t)&value;
	threshbuf.priority = NETMGT_LOW_PRIORITY;
	 
#endif
	if (!request_event(count, interval, timeout, &threshbuf, 
	    group, optstring, key, flags )) {
		NETMGT_DBG("Cannot start event reporting.\n");
        }


#else	/* request data report */


	if (!request_data(count, interval, timeout, 
	    group, optstring, key, flags )) {
		NETMGT_DBG("Cannot start data reporting.\n");
        }

#endif

}

/*
 * Chapter 4.9: Sample Code
 */

/*
 *  request_event - send event report request to agent
 *	returns request timestamp if successful; otherwise NULL
 */
struct timeval *
request_event(count, interval, timeout, threshold, group, optstring, key, flags)
    u_int count;		/* report count */
    struct timeval interval;	/* reporting interval */
    struct timeval timeout;	/* rpc timeout */
    Netmgt_thresh *threshold;	/* event threshold */
    char *group;		/* group of statistics */
    char *optstring;		/* optstring to the  agent */
    char *key;			/* key for table group */
    u_int flags;		/* restart */
{

	Netmgt_arg 	arg;
	struct rpcent 	*agent_rpc;	/* rpc entry for the agent */
	struct timeval	*timestamp;	/* request timestamp */

	u_long		agent_prog;	/* agent's RPC program number */
	u_long		rendez_prog;	/* rendez vous RPC program number */

	NETMGT_DBG("requesting events \n");


	/* declare managed object instance */
	if (!netmgt_set_instance(target, group, key)) {
		NETMGT_DBG("set_instance failed for %s: %s\n",
		    target, netmgt_sperror());
		return (struct timeval *) NULL;
	}

#ifndef SNMP_AGENT
	/* get the agent's RPC info */
	agent_rpc = getrpcbyname(agentname);
	if (!agent_rpc) {
		NETMGT_DBG("can't get RPC program number for %s\n", agentname);
		return (struct timeval *) NULL;
	}
	agent_prog = agent_rpc->r_number;

#else 
	agent_prog = SNMP_RPC_NUM;
	/* set SNMP arguments */
	set_snmp_args();
#endif


	/* pass optional arguments */
	if (optstring) {
		(void) strcpy(arg.name, NETMGT_OPTSTRING);
		arg.type = NETMGT_STRING;
		arg.length = strlen(optstring) + 1;
		arg.value = optstring;
		if (!netmgt_set_argument(&arg)) {
			NETMGT_DBG ("netmgt_set_argument failed: %s \n",
			    netmgt_sperror());
			return (struct timeval *) NULL;
		}
	}

	/* set threshold */
	if (!netmgt_set_threshold(threshold)) {
		NETMGT_DBG("netmgt_set_threshold failed: %s\n", 
		    netmgt_sperror());
		return (struct timeval *) NULL;
	}

	/* send the event report to the event dispatcher */
	rendez_prog = NETMGT_EVENT_PROG;

	/* start the event report request */
	timestamp = netmgt_request_events(agent_host, agent_prog, 
	    NETMGT_VERS, event_host, rendez_prog, NETMGT_VERS,
	    count, interval, timeout, flags);

	if (!timestamp)
		NETMGT_DBG("Can't request event report: %s\n",netmgt_sperror());

	return timestamp;

}

/*
 *  request_data - send data report request to agent
 *	returns request timestamp if successful; otherwise NULL
 */
struct timeval *
request_data(count, interval, timeout, group, optstring, key, flags)
    u_int count;		/* report count */
    struct timeval interval;	/* reporting interval */
    struct timeval timeout;	/* rpc timeout */
    char *group;		/* group of statistics */
    char *optstring;		/* optstring to the  agent */
    char *key;			/* key for table group */
    u_int flags;		/* restart */
{

	Netmgt_arg 	arg;
	struct rpcent 	*agent_rpc;	/* rpc entry for the agent */
	struct timeval	*timestamp;	/* request timestamp */

	u_long		agent_prog;	/* agent's RPC program number */
	u_long		rendez_prog;	/* rendez vous RPC program number */

	NETMGT_DBG("requesting data \n");


	/* declare managed object instance */
	if (!netmgt_set_instance(target, group, key)) {
		NETMGT_DBG("set_instance failed for %s: %s\n",
		    target, netmgt_sperror());
		return (struct timeval *) NULL;
	}

#ifndef SNMP_AGENT
	/* get the agent's RPC info */
	agent_rpc = getrpcbyname(agentname);
	if (!agent_rpc) {
		NETMGT_DBG("can't get RPC program number for %s\n", agentname);
		return (struct timeval *) NULL;
	}
	agent_prog = agent_rpc->r_number;

#else 
	agent_prog = SNMP_RPC_NUM;
	/* set SNMP arguments */
	set_snmp_args();
#endif

	/* pass optional arguments */
	if (optstring) {
		(void) strcpy(arg.name, NETMGT_OPTSTRING);
		arg.type = NETMGT_STRING;
		arg.length = strlen(optstring) + 1;
		arg.value = optstring;
		if (!netmgt_set_argument(&arg)) {
			NETMGT_DBG ("netmgt_set_argument failed: %s \n",
			    netmgt_sperror());
			return (struct timeval *) NULL;
		}
	}


	/* send the data report to the logger */
	rendez_prog = NETMGT_LOGGER_PROG;

	/* start the event report request */
	timestamp = netmgt_request_data(agent_host, agent_prog, 
	    NETMGT_VERS, event_host, rendez_prog, NETMGT_VERS,
	    count, interval, timeout, flags);

	if (!timestamp)
		NETMGT_DBG("Can't request event report: %s\n",netmgt_sperror());

	return timestamp;

}

/*
 * chapter 4.2: Optional Arguments
 */

bool_t
set_snmp_args()
{
	/* argument for setting options */
	Netmgt_arg 	option;

	/* read community string */
	char	*community= "community string";

	/* SNMP retry interval (in seconds) */
	int 	retry = 10;

	/* OID list for instances (sysObjectID and sysUpTime) */
	char 	*get_list = "1.3.6.1.2.1.1.2.0 1.3.6.1.2.1.1.3.0";

 
	NETMGT_DBG("setting snmp arguments \n");

	/* send SNMP read community */
        /*	not needed if read access is free 
	strcpy(option.name, "na.snmp-read-community");
	option.type = NETMGT_STRING;
	option.length = strlen(community) + 1;
	option.value = community;
	if  (!netmgt_set_argument(&option)) {
		NETMGT_DBG ("netmgt_set_argument failed: %s \n",
		    netmgt_sperror());
		return (FALSE);
	}
	*/


	/* send SNMP retry interval */
	strcpy(option.name, "na.snmp.retry");
	option.type = NETMGT_INT;
	option.length = sizeof (int);
	option.value = (char *) &retry;
	if  (!netmgt_set_argument(&option)) {
		NETMGT_DBG ("netmgt_set_argument failed: %s \n",
		    netmgt_sperror());
		return (FALSE);
	}
 
#ifndef REQUEST_EVENT_REPORT
	/* send OID list to request data only for certain attributes */
	strcpy(option.name, "na.snmp-get-list");
	option.type = NETMGT_STRING;
	option.length = strlen(get_list) + 1;
	option.value = get_list;
	if  (!netmgt_set_argument(&option)) {
		NETMGT_DBG ("netmgt_set_argument failed: %s \n",
		    netmgt_sperror());
		return (FALSE);
	}
#endif

}

