/* Copyright 1994 Sun Microsystems, Inc. All Rights Reserved. */
/* Copyright 1993 Sun Microsystems, Inc. All Rights Reserved. */

/**************************************************************************
 *  File:	include/netmgt/netmgt_request.h
 *
 *  Author:	Lynn Monsanto, Sun Microsystems Inc.
 *
 *  Copyright (c) 1989 Sun Microsystems, Inc.  All Rights Reserved.
 *  Sun considers its source code as an unpublished, proprietary trade 
 *  secret, and it is available only under strict license provisions.  
 *  This copyright notice is placed here only to protect Sun in the event
 *  the source is deemed a published work.  Dissassembly, decompilation, 
 *  or other means of reducing the object code to human readable form is 
 *  prohibited by the license agreement under which this code is provided
 *  to the user or company in possession of this copy.
 * 
 *  RESTRICTED RIGHTS LEGEND: Use, duplication, or disclosure by the 
 *  Government is subject to restrictions as set forth in subparagraph 
 *  (c)(1)(ii) of the Rights in Technical Data and Computer Software 
 *  clause at DFARS 52.227-7013 and in similar clauses in the FAR and 
 *  NASA FAR Supplement.
 *
 *  SCCSID:	@(#)netmgt_request.h  1.20  94/11/30
 *
 *  Comments:	request context definitions
 *
 **************************************************************************
 */

#ifndef	_netmgt_request_h
#define _netmgt_request_h

/* additional request types */
#define NETMGT_TERMINATE_REQUEST  (u_int)13     /* terminate a request */
#define NETMGT_TERMINATE_REQUEST2 (u_int)14	/* terminate manager requests */
#define NETMGT_CONFIRM_REQUEST	  (u_int)15     /* confirm a request */
#define NETMGT_REQUEST_TERMINATED (u_int)16     /* a request terminated */

/* additional request flags */
#define NETMGT_DONT_DISPATCH	  (1 << 10) 	/* don't dispatch request */

/* basic request information */
typedef struct 
{
  struct in_addr manager_addr ;  /* manager IP address */
  struct timeval request_time ;  /* request timestamp */
  u_int type ;			 /* request type */
  u_int manager_id ;             /* manager ID */ 
  u_int flags ;			 /* request flags */
  u_int priority ;		 /* request priority */
  struct in_addr rendez_addr ;	 /* rendezvous IP address */
  u_long rendez_prog ;		 /* rendezvous RPC program number */
  u_long rendez_vers ;		 /* rendezvous RPC version number */
  u_long proto;			 /* transport protocol */
  struct timeval interval ;	 /* reporting interval */
  u_int count ;			 /* reporting count */
  char system [NETMGT_NAMESIZ] ; /* system */
  char group [NETMGT_NAMESIZ] ;  /* group */
  char key [NETMGT_NAMESIZ] ;    /* key */
  u_int num_arguments ;		 /* number of optional arguments */
  u_int num_thresholds ;	 /* number of event thresholds */
  u_int num_setvals ;	 	 /* number of set arguments */
} 	Netmgt_request ;

#define NETMGT_VALUESIZ		NETMGT_NAMESIZ

/* event threshold */
typedef struct
{
  char name [NETMGT_NAMESIZ] ;	        /* attribute name */
  u_int type ;				/* attribute type */
  u_int relop ;				/* relational operator */
  u_int thresh_len ;			/* threshold value length */
  u_char thresh_val [NETMGT_VALUESIZ] ; /* threshold value buffer */
  u_int prev_len ;			/* previous attribute value length */
  u_char prev_val [NETMGT_VALUESIZ];	/* previous attribute value buffer */
  u_int priority ;			/* event priority */
}      Netmgt_threshold ;

/* optional argument */
typedef struct
{
  char name [NETMGT_NAMESIZ] ;	        /* argument name */
  u_int type ;				/* argument type */
  u_int length ;			/* argument length */
  u_char value [NETMGT_VALUESIZ] ;	/* argument value buffer */
}      Netmgt_argument ;


#if defined (__cplusplus)

extern "C" {

bool_t netmgt_init_child (void) ;

bool_t netmgt_clear_arguments (void) ;

bool_t netmgt_save_argument (char *arg_name, Netmgt_argument *arg);

bool_t netmgt_restore_argument (Netmgt_argument *arg);

bool_t netmgt_save_request (Netmgt_request *request) ;

bool_t netmgt_restore_request (Netmgt_request *request) ;

bool_t netmgt_clear_thresholds (void) ;

bool_t netmgt_save_threshold (char *thresh_name, Netmgt_threshold *thresh);

bool_t netmgt_restore_threshold (Netmgt_threshold *thresh);

bool_t netmgt_delete_request (Netmgt_request *request) ;

bool_t netmgt_delete_request2 (Netmgt_request *request) ;

bool_t netmgt_get_request2confirm (Netmgt_request *request) ;

bool_t netmgt_get_request2terminate (Netmgt_request *request) ;

bool_t netmgt_kill_request2 (char *agent_name, 
			     u_long agent_prog, 
			     u_long agent_vers, 
			     char *manager_name, 
			     u_int manager_id, 
			     struct timeval timeout) ;
}

#else /* Sun C */

extern bool_t netmgt_init_child () ;

extern bool_t netmgt_clear_arguments () ;

extern bool_t netmgt_save_argument () ;

extern bool_t netmgt_restore_argument () ;

extern bool_t netmgt_save_request () ;

extern bool_t netmgt_restore_request () ;

extern bool_t netmgt_clear_thresholds () ;

extern bool_t netmgt_save_threshold () ;

extern bool_t netmgt_restore_threshold () ;

extern bool_t netmgt_delete_request () ;

extern bool_t netmgt_delete_request2 () ;

extern bool_t netmgt_get_request2confirm () ;

extern bool_t netmgt_get_request2terminate () ;

extern bool_t netmgt_kill_request2 () ;

#endif defined (__cplusplus)

#endif _netmgt_request_h 

